package data_manage

import (
	"fmt"
	"github.com/shopspring/decimal"
	"hongze/hongze_task/utils"
	"rdluck_tools/orm"
	"strconv"
	"strings"
	"time"
)

type EdbInfoCalculateTbz struct {
	EdbInfoCalculateTbzId int       `orm:"column(edb_info_calculate_tbz_id);pk"`
	EdbInfoId             int       `description:"指标id"`
	EdbCode               string    `description:"指标编码"`
	FromEdbInfoId         int       `description:"计算指标id"`
	FromEdbCode           string    `description:"计算指标编码"`
	FromEdbName           string    `description:"计算指标名称"`
	FromSource            int       `description:"计算指标来源"`
	FromSourceName        string    `description:"计算指标来源名称"`
	FromTag               string    `description:"来源指标标签"`
	Sort                  int       `description:"计算指标名称排序"`
	CreateTime            time.Time `description:"创建时间"`
	ModifyTime            time.Time `description:"修改时间"`
}

func TbzDiv(a, b float64) float64 {
	af := decimal.NewFromFloat(float64(a))
	bf := decimal.NewFromFloat(float64(b))
	val, _ := af.Div(bf).Float64()
	val = val - 1
	return val
}

//刷新同比值数据
func RefreshCalculateTbz(edbInfoId int, fromEdbInfo *EdbInfo, edbCode, startDate, endDate string) (err error) {
	o := orm.NewOrm()
	o.Using("data")
	o.Begin()
	defer func() {
		if err != nil {
			o.Rollback()
		} else {
			o.Commit()
		}
	}()

	if err != nil {
		return
	}
	edbInfoIdStr := strconv.Itoa(edbInfoId)
	//计算数据

	//计算数据
	var condition string
	var pars []interface{}
	condition += " AND edb_info_id=? "
	pars = append(pars, edbInfoId)

	if startDate != "" {
		condition += " AND data_time>=? "
		pars = append(pars, startDate)
	}
	if endDate != "" {
		condition += " AND data_time<=? "
		pars = append(pars, endDate)
	}

	dataList, err := GetEdbDataListAll(condition, pars, fromEdbInfo.Source, 0)
	if err != nil {
		return err
	}
	var dateArr []string
	dataMap := make(map[string]*EdbInfoSearchData)
	for _, v := range dataList {
		dateArr = append(dateArr, v.DataTime)
		dataMap[v.DataTime] = v
	}

	addSql := ` INSERT INTO edb_data_calculate_tbz(edb_info_id,edb_code,data_time,value,create_time,modify_time,status,data_timestamp) values `
	var isAdd bool
	for ak, av := range dateArr {
		fmt.Println(ak, av)
		currentItem := dataMap[av]
		if currentItem != nil {
			//当前日期
			currentDate, err := time.Parse(utils.FormatDate, av)
			if err != nil {
				return err
			}
			//上一年的日期
			preDate := currentDate.AddDate(-1, 0, 0)
			preDateStr := preDate.Format(utils.FormatDate)
			if findItem, ok := dataMap[preDateStr]; ok { //上一年同期找到
				//dataTime, _ := time.Parse(utils.FormatDate, date)
				timestamp := currentDate.UnixNano() / 1e6
				timestampStr := fmt.Sprintf("%d", timestamp)
				val := TbzDiv(currentItem.Value, findItem.Value)

				count, err := GetEdbDataCalculateTbzByCodeAndDate(edbCode, av)
				if err != nil && err.Error() != utils.ErrNoRow() {
					return err
				}
				if count <= 0 {
					addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
					isAdd = true
				} else {
					err = ModifyEdbDataCalculateTbz(int64(edbInfoId), av, val)
					if err != nil {
						return err
					}
				}
				utils.FileLog.Info("同期找到:" + av + ";" + preDateStr)
				continue
			} else {
				if fromEdbInfo.Frequency == "月度" { //向上和向下，各找一个月
					nextDateDay := preDate.AddDate(0, 1, 0)
					nextDateDayStr := nextDateDay.Format(utils.FormatDate)

					preDateDay := preDate.AddDate(0, -1, 0)
					preDateDayStr := preDateDay.Format(utils.FormatDate)

					for i := 0; i <= 6; i++ {
						if i >= 1 {
							nextDateDay = nextDateDay.AddDate(0, 0, i)
							nextDateDayStr = nextDateDay.Format(utils.FormatDate)
						}
						if findItem, ok := dataMap[nextDateDayStr]; ok { //上一年同期->下一个月找到
							timestamp := currentDate.UnixNano() / 1e6
							timestampStr := fmt.Sprintf("%d", timestamp)
							val := TbzDiv(currentItem.Value, findItem.Value)

							count, err := GetEdbDataCalculateTbzByCodeAndDate(edbCode, av)
							if err != nil && err.Error() != utils.ErrNoRow() {
								return err
							}
							if count <= 0 {
								addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
								isAdd = true
							} else {
								err = ModifyEdbDataCalculateTbz(int64(edbInfoId), av, val)
								if err != nil {
									return err
								}
							}
							break
						} else {
							if i >= 1 {
								preDateDay = preDate.AddDate(0, 0, -i)
								preDateDayStr = nextDateDay.Format(utils.FormatDate)
							}
							if findItem, ok := dataMap[preDateDayStr]; ok { //上一年同期->上一个月找到
								timestamp := currentDate.UnixNano() / 1e6
								timestampStr := fmt.Sprintf("%d", timestamp)
								val := TbzDiv(currentItem.Value, findItem.Value)

								count, err := GetEdbDataCalculateTbzByCodeAndDate(edbCode, av)
								if err != nil && err.Error() != utils.ErrNoRow() {
									return err
								}
								if count <= 0 {
									addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
									isAdd = true
								} else {
									err = ModifyEdbDataCalculateTbz(int64(edbInfoId), av, val)
									if err != nil {
										return err
									}
								}
							}
							break
						}
					}
				} else if fromEdbInfo.Frequency == "季度" || fromEdbInfo.Frequency == "年度" {
					if findItem, ok := dataMap[preDateStr]; ok { //上一年同期->下一个月找到
						timestamp := currentDate.UnixNano() / 1e6
						timestampStr := fmt.Sprintf("%d", timestamp)
						val := TbzDiv(currentItem.Value, findItem.Value)

						count, err := GetEdbDataCalculateTbzByCodeAndDate(edbCode, av)
						if err != nil && err.Error() != utils.ErrNoRow() {
							return err
						}
						if count <= 0 {
							addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
							isAdd = true
						} else {
							err = ModifyEdbDataCalculateTbz(int64(edbInfoId), av, val)
							if err != nil {
								return err
							}
						}
						break
					}
				} else {
					nextDateDay := preDate.AddDate(0, 0, 1)
					nextDateDayStr := nextDateDay.Format(utils.FormatDate)

					preDateDay := preDate.AddDate(0, 0, -1)
					preDateDayStr := preDateDay.Format(utils.FormatDate)

					for i := 0; i < 35; i++ {
						if i >= 1 {
							nextDateDay = nextDateDay.AddDate(0, 0, i)
							nextDateDayStr = nextDateDay.Format(utils.FormatDate)
						}
						if findItem, ok := dataMap[nextDateDayStr]; ok { //上一年同期->下一个月找到
							timestamp := currentDate.UnixNano() / 1e6
							timestampStr := fmt.Sprintf("%d", timestamp)
							val := TbzDiv(currentItem.Value, findItem.Value)

							count, err := GetEdbDataCalculateTbzByCodeAndDate(edbCode, av)
							if err != nil && err.Error() != utils.ErrNoRow() {
								return err
							}
							if count <= 0 {
								addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
								isAdd = true
							} else {
								err = ModifyEdbDataCalculateTbz(int64(edbInfoId), av, val)
								if err != nil {
									return err
								}
							}
							break
						} else {
							if i >= 1 {
								preDateDay = preDate.AddDate(0, 0, -i)
								preDateDayStr = nextDateDay.Format(utils.FormatDate)
							}
							if findItem, ok := dataMap[preDateDayStr]; ok { //上一年同期->上一个月找到
								timestamp := currentDate.UnixNano() / 1e6
								timestampStr := fmt.Sprintf("%d", timestamp)
								val := TbzDiv(currentItem.Value, findItem.Value)

								count, err := GetEdbDataCalculateTbzByCodeAndDate(edbCode, av)
								if err != nil && err.Error() != utils.ErrNoRow() {
									return err
								}
								if count <= 0 {
									addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
									isAdd = true
								} else {
									err = ModifyEdbDataCalculateTbz(int64(edbInfoId), av, val)
									if err != nil {
										return err
									}
								}
							}
							break
						}
					}
				}
			}
		}
	}
	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		_, err = o.Raw(addSql).Exec()
		if err != nil {
			return err
		}
	}
	return
}

type EdbInfoCalculateTbzDetail struct {
	EdbInfoCalculateTbzId int       `orm:"column(edb_info_calculate_tbz_id);pk"`
	EdbInfoId             int       `description:"指标id"`
	EdbCode               string    `description:"指标编码"`
	FromEdbInfoId         int       `description:"计算指标id"`
	FromEdbCode           string    `description:"计算指标编码"`
	FromEdbName           string    `description:"计算指标名称"`
	FromSource            int       `description:"计算指标来源"`
	FromSourceName        string    `description:"计算指标来源名称"`
	FromTag               string    `description:"来源指标标签"`
	Sort                  int       `description:"计算指标名称排序"`
	CreateTime            time.Time `description:"创建时间"`
	ModifyTime            time.Time `description:"修改时间"`
	StartDate             string    `description:"开始日期"`
	EndDate               string    `description:"结束日期"`
}

func GetEdbInfoCalculateTbzDetail(edbInfoId int) (item *EdbInfoCalculateTbzDetail, err error) {
	o := orm.NewOrm()
	o.Using("data")
	sql := ` SELECT a.*,b.start_date,b.end_date FROM edb_info_calculate_tbz AS a
			INNER JOIN edb_info AS b ON a.from_edb_info_id=b.edb_info_id
			WHERE a.edb_info_id=? `
	err = o.Raw(sql, edbInfoId).QueryRow(&item)
	return
}

func GetEdbDataCalculateTbzByCodeAndDate(edbCode string, startDate string) (count int, err error) {
	o := orm.NewOrm()
	o.Using("data")
	sql := ` SELECT COUNT(1) AS count FROM edb_data_calculate_tbz WHERE edb_code=? AND data_time=? `
	err = o.Raw(sql, edbCode, startDate).QueryRow(&count)
	return
}

func ModifyEdbDataCalculateTbz(edbInfoId int64, dataTime string, value float64) (err error) {
	o := orm.NewOrm()
	o.Using("data")
	sql := ` UPDATE edb_data_calculate_tbz SET value=?,modify_time=NOW() WHERE edb_info_id=? AND data_time=? `
	_, err = o.Raw(sql, value, edbInfoId, dataTime).Exec()
	return
}
